package com.sample;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.DecisionTableConfiguration;
import org.drools.builder.DecisionTableInputType;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.command.Command;
import org.drools.command.CommandFactory;
import org.drools.compiler.DecisionTableFactory;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

/**
 * Class to reproduce bug in XLS tables working with utf-8. Rule names are crippled.
 * CSV decision tables in utf8 work fine. Possible bug in used xls parser.
 * 
 * @encoding utf-8
 * @author jsvitak@redhat.com
 *
 */
public class UnicodeTest {

    public static void main(String [] args) throws FileNotFoundException {
        
        //KnowledgeBuilder kbuilder = getKnowledgeBuilder("unicode.csv", DecisionTableInputType.CSV);
        KnowledgeBuilder kbuilder = getKnowledgeBuilder("unicode.xls", DecisionTableInputType.XLS);
        
        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
        
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
        
        List <Command<?>> commands = new ArrayList<Command<?>>();
        List<Člověk> dospělí = new ArrayList<Člověk>();
        commands.add(CommandFactory.newSetGlobal("dospělí", dospělí));
        Člověk Řehoř = new Člověk();
        Řehoř.setVěk(30);
        Řehoř.setJméno("Řehoř");
        commands.add(CommandFactory.newInsert(Řehoř));
        commands.add(CommandFactory.newFireAllRules());
        ksession.execute(CommandFactory.newBatchExecution(commands));
        
        assertNotNull(kbase.getRule("com.sample", "pokusné pravidlo rozhodovací tabulky"));
        assertEquals(dospělí.size(), 1);
        assertEquals(dospělí.iterator().next().getJméno(), "Řehoř");
         
    }
    
    /**
     * 
     * @param fileName "unicode.xls" or "unicode.csv"
     * @param type
     * @return
     * @throws FileNotFoundException
     */
    private static KnowledgeBuilder getKnowledgeBuilder(String fileName, DecisionTableInputType type) throws FileNotFoundException {
        DecisionTableConfiguration dtconf = KnowledgeBuilderFactory.newDecisionTableConfiguration();
        dtconf.setInputType(type);
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newFileResource(new File("test-resources", fileName)), ResourceType.DTABLE, dtconf);
        System.out.println(kbuilder.getErrors().toString());
        System.out.println(DecisionTableFactory.loadFromInputStream(new FileInputStream(new File("test-resources", fileName)), dtconf));
        return kbuilder;
    }
    
    // czech person
    public static class Člověk {
        // age
        private int věk;
        // name
        private String jméno;
        public void setVěk(int věk) {
            this.věk = věk;
        }
        public int getVěk() {
            return věk;
        }
        public void setJméno(String jméno) {
            this.jméno = jméno;
        }
        public String getJméno() {
            return jméno;
        }
    }
}
